import java.lang.reflect.Field;
import junit.framework.*;
import java.util.*;

/**
 * The test class ParkingControlTest.
 *
 * @author  Nicola Asuni
 * @version 2005-12-10
 */
public class ParkingControlTest extends TestCase {

    
    private ArrayList<Ticket> tickets;
    private long lastTicket;
    private ParkingControl PC;
    Calendar now;
    
    //private int places;
    //private long lastTicket;
    
    public static void main (String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        TestSuite newSuite = new TestSuite();
        newSuite.addTest(new ParkingControlTest("testAddCar"));
        newSuite.addTest(new ParkingControlTest("testExit"));
        newSuite.addTest(new ParkingControlTest("testGetPrice"));
        newSuite.addTest(new ParkingControlTest("testPayment"));
        newSuite.addTest(new ParkingControlTest("testHowMany"));
        newSuite.addTest(new ParkingControlTest("testAverageRevenue"));
        newSuite.addTest(new ParkingControlTest("testAverageOccupancy"));
        return newSuite;
    }
    
    public ParkingControlTest(String s) {
        super(s);
    }

    /**
     * Default constructor for test class ParkingControlTest
     */
    public ParkingControlTest() {
    }

    private Date setParkTime(int diffday, int hour, int minute) {
        Calendar cal = new GregorianCalendar();
        cal.add(Calendar.DAY_OF_MONTH, -diffday);
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.MINUTE, minute);
        return cal.getTime();
    }
    
    private void setTickets(int diffday) {
       // data fixture for testing
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 8, 7), setParkTime(diffday, 8, 52), 75));
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 9, 24), setParkTime(diffday, 11, 30), 210));
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 10, 6), setParkTime(diffday, 10, 53), 78)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 10, 17), setParkTime(diffday, 14, 22), 340)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 11, 26), setParkTime(diffday, 18, 37), 519)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 12, 5), setParkTime(diffday, 13, 34), 120)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 13, 13), setParkTime(diffday, 16, 31), 165)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 14, 0), setParkTime(diffday, 15, 0), 50)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 15, 4), setParkTime(diffday, 15, 55), 43)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 16, 3), setParkTime(diffday, 17, 56), 141)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 16, 16), setParkTime(diffday, 19, 23), 275)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 17, 22), setParkTime(diffday, 19, 32), 217)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 18, 2), setParkTime(diffday, 18, 57), 92)); 
       tickets.add(new Ticket(lastTicket++, setParkTime(diffday, 19, 1), setParkTime(diffday, 19, 58), 95)); 
    }

    /**
     * Sets up the test fixture.
     *
     * Called before every test case method.x
     */
    protected void setUp() {
        
        lastTicket = 1;
        tickets = new ArrayList<Ticket>();
        
        now = new GregorianCalendar();
        int nowday = now.get(Calendar.DAY_OF_MONTH);
        
        // -2 mesi
        setTickets(nowday + 35);
        setTickets(nowday + 33);
        
        // mese precedente
        setTickets(nowday + 25);
        setTickets(nowday + 20);
        setTickets(nowday + 10);
        
        // giorno precedente
        setTickets(1);
        
        PC = new ParkingControl(20);
        PrivateAccessor.setPrivateField(PC, "tickets", tickets);
    }

    /**
     * Tears down the test fixture.
     *
     * Called after every test case method.
     */
    protected void tearDown() {
        tickets.clear();
    }
    
    // -- test methods ---
    
    public void testAddCar() {
        ParkingControl PC2 = new ParkingControl(2);
        long ticketnum = PC2.addCar();
        assertTrue(ticketnum > 0);
        assertEquals(ticketnum + 1, PC2.addCar());
        assertEquals(0, PC2.addCar());
    }
    
    public void testExit() {
       Calendar curdate = new GregorianCalendar(); // get current date
       Calendar indate = new GregorianCalendar(); // get current date
       
       indate.set(Calendar.HOUR_OF_DAY, 8);
       indate.set(Calendar.MINUTE, 8);
       
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertTrue(PC.exit(lastTicket-1, indate.getTime()));
       
       // regola dei 2 minuti
       
       indate.set(Calendar.MINUTE, 7);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertTrue(PC.exit(lastTicket-1, indate.getTime()));
       
       indate.set(Calendar.MINUTE, 9);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertTrue(PC.exit(lastTicket-1, indate.getTime()));
       
       indate.set(Calendar.MINUTE, 5);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertFalse(PC.exit(lastTicket-1, indate.getTime()));
       
       indate.set(Calendar.MINUTE, 11);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertFalse(PC.exit(lastTicket-1, indate.getTime()));
       
       // nessun pagamento
       
       indate.set(Calendar.MINUTE, 8);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), null, 1));
       assertFalse(PC.exit(lastTicket-1, indate.getTime()));
       
       // regola dei 10 minuti
       
       curdate.add(Calendar.MINUTE, -20);
       indate.set(Calendar.MINUTE, 8);
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 8), setParkTime(0, curdate.get(Calendar.HOUR_OF_DAY), curdate.get(Calendar.MINUTE)), 1));
       assertFalse(PC.exit(lastTicket-1, indate.getTime()));  
    }
    
    public void testGetPrice() {
       assertEquals(75, PC.getPrice(1));
       assertEquals(210, PC.getPrice(2));
       assertEquals(78, PC.getPrice(3));
       assertEquals(340, PC.getPrice(4));
       assertEquals(519, PC.getPrice(5));
       assertEquals(120, PC.getPrice(6));
       assertEquals(165, PC.getPrice(7));
       assertEquals(50, PC.getPrice(8));
       assertEquals(43, PC.getPrice(9));
       assertEquals(141, PC.getPrice(10));
       assertEquals(275, PC.getPrice(11));
       assertEquals(217, PC.getPrice(12));
       assertEquals(92, PC.getPrice(13));
       assertEquals(95, PC.getPrice(14));
    }

    public void testPayment() {
       long firstticket = lastTicket;
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 7), null, 0));
       PC.payment(firstticket);
       // il metodo payment() deve impostare la data di uscita
       assertNotNull(tickets.get((int)firstticket-1).getTimeOut()); 
    }
    
    public void testHowMany() {
       long firstticket = lastTicket;
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 8, 7), null, 0));
       assertEquals(1, PC.howMany());
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 9, 24), setParkTime(0, 11, 30), 210));
       assertEquals(1, PC.howMany());
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 10, 6), null, 0));
       assertEquals(2, PC.howMany());
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 10, 17), setParkTime(0, 14, 22), 340)); 
       assertEquals(2, PC.howMany());
       tickets.add(new Ticket(lastTicket++, setParkTime(0, 11, 26), null, 0));
       assertEquals(3, PC.howMany());
    }
    
    public void testAverageRevenue() {
        Calendar curdate = new GregorianCalendar(); // get current date
        curdate.add(Calendar.MONTH, -1); // get previous month
        int monthdays = curdate.getMaximum(Calendar.DAY_OF_MONTH); // number of days in previous month
        if (now.get(Calendar.DAY_OF_MONTH) == 1) {
            assertEquals(Math.round((float)2420 * 4 / monthdays), PC.averageRevenue());
        } else {
            assertEquals(Math.round((float)2420 * 3 / monthdays), PC.averageRevenue());
        }
    }
    
    public void testAverageOccupancy() {
        int[] occupancy = PC.averageOccupancy();
        assertEquals(1, occupancy[0]); // ore 8-9
        assertEquals(1, occupancy[1]); // ore 9-10
        assertEquals(3, occupancy[2]); // ore 10-11
        assertEquals(2, occupancy[3]); // ore 11-12
        assertEquals(3, occupancy[4]); // ore 12-13
        assertEquals(3, occupancy[5]); // ore 13-14
        assertEquals(3, occupancy[6]); // ore 14-15
        assertEquals(3, occupancy[7]); // ore 15-16
        assertEquals(3, occupancy[8]); // ore 16-17
        assertEquals(4, occupancy[9]); // ore 17-18
        assertEquals(4, occupancy[10]); // ore 18-19
        assertEquals(2, occupancy[11]); // ore 19-20  
    }
    
}




